package randomcitygenerator;

import java.awt.Dimension;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;

/**
 *
 * @author Martin
 */
public class RandomCityGenerator {

    private String cityName;
    private Dimension citySize;

    private RandomMapCreator randomMapCreator;
    private CityBuilder cityBuilder;

    private URL cityTileSet;

    public RandomCityGenerator() {
        this.cityTileSet = this.getClass().getResource("/randomcitygenerator/src/city_tileset.png");
    }

    private void readData(String[] args) {
        try {
            if (Integer.parseInt(args[1]) <= 0 || Integer.parseInt(args[2]) <= 0) {
                System.exit(0);
            }
            this.cityName = args[0];
            this.citySize = new Dimension(
                    Integer.parseInt(args[1]),
                    Integer.parseInt(args[2])
            );
        } catch (Exception ex) {
            System.exit(0);
        }
    }

    private void createMap() {
        long l = System.nanoTime();
        System.out.print("Map generating: ");

        this.randomMapCreator = new RandomMapCreator(citySize);
        this.randomMapCreator.generateMap();

        System.out.println((double) ((System.nanoTime() - l) / 1e9) + "s");
    }

    private void createCityImage() {
        long l = System.nanoTime();
        System.out.print("Rendering: ");

        this.cityBuilder = new CityBuilder();
        this.cityBuilder.init(cityTileSet, 6, 3);
        this.cityBuilder.build(
                this.randomMapCreator.MAP,
                this.citySize
        );

        System.out.println((double) ((System.nanoTime() - l) / 1e9) + "s");
    }

    private void saveCityImage() {
        long l = System.nanoTime();
        System.out.print("Saving: ");

        try {
            ImageIO.write(this.cityBuilder.cityImage, "png", new File(this.cityName + ".png"));
        } catch (IOException ex) {
            Logger.getLogger(RandomCityGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }

        System.out.println((double) ((System.nanoTime() - l) / 1e9) + "s");
    }

    private void generateData() {
        long l = System.nanoTime();
        System.out.print("Data Generating: ");
        
        try {
            BufferedWriter buffer = new BufferedWriter(new FileWriter(this.cityName + ".dat"));
            String[] arr = this.randomMapCreator.generateData().split("/");
            for (String s : arr) {
                buffer.write(s);
                buffer.newLine();
            }
            buffer.close();
        } catch (IOException ex) {
            Logger.getLogger(RandomCityGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        System.out.println((double) ((System.nanoTime() - l) / 1e9) + "s");
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        RandomCityGenerator randomCityGenerator = new RandomCityGenerator();
        //read data: name widht height
        randomCityGenerator.readData(args);
        //generate random map
        randomCityGenerator.createMap();
        //create image of city
        randomCityGenerator.createCityImage();
        //sava image fo city
        randomCityGenerator.saveCityImage();
        //generate data
        randomCityGenerator.generateData();
    }

}
