package randomcitygenerator;

import java.awt.Dimension;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author Martin
 */
public class RandomMapCreator extends BlockInfo {

    private final Dimension mapSize;

    private final Random rnd = new Random();

    private final ArrayList<Point> allPlaces;

    public int[][] MAP;

    public RandomMapCreator(Dimension d) {
        this.mapSize = d;
        this.allPlaces = new ArrayList<>();
    }

    /**
     * Generate Map
     * ###################################################################
     */
    public void generateMap() {
        if (mapSize == null) {
            return;
        }
        MAP = new int[this.mapSize.width][this.mapSize.height];

        for (int x = 0; x < mapSize.width; x++) {
            for (int y = 0; y < mapSize.height; y++) {
                MAP[x][y] = EMPTY;
                allPlaces.add(new Point(x, y));
            }
        }

        placeBuildings(0.45f);

        roadPlacing();

        deleteEmptyPlaces();

        for (int i = 0; i < 2; i++) {
            for (int x = 0; x < mapSize.width; x++) {
                for (int y = 0; y < mapSize.height; y++) {
                    if (MAP[x][y] != BUILDING) {
                        MAP[x][y] = EMPTY;
                    }
                }
            }
            roadPlacing();
            correction();
        }
    }

    private void correction() {
        for (int x = 0; x < mapSize.width; x++) {
            for (int y = 0; y < mapSize.height; y++) {
                subCorrection1(x, y);
            }
        }

        for (int x = 0; x < mapSize.width; x++) {
            for (int y = 0; y < mapSize.height; y++) {
                subCorrection2(x, y);
            }
        }
    }

    private void deleteEmptyPlaces() {
        for (int x = 0; x < mapSize.width; x++) {
            for (int y = 0; y < mapSize.height; y++) {
                if (MAP[x][y] == EMPTY) {
                    boolean b = true;
                    try {
                        if (MAP[x + 1][y] == BUILDING && b) {
                            MAP[x + 1][y] = EMPTY;
                            b = true;
                        }
                    } catch (Exception ex) {
                    }
                    try {
                        if (MAP[x - 1][y] == BUILDING && b) {
                            MAP[x - 1][y] = EMPTY;
                            b = true;
                        }
                    } catch (Exception ex) {
                    }
                    try {
                        if (MAP[x][y + 1] == BUILDING && b) {
                            MAP[x][y + 1] = EMPTY;
                            b = true;
                        }
                    } catch (Exception ex) {
                    }
                    try {
                        if (MAP[x][y - 1] == BUILDING && b) {
                            MAP[x][y - 1] = EMPTY;
                            b = true;
                        }
                    } catch (Exception ex) {
                    }
                }
            }
        }
    }

    private int placedBuildings = 0;   
    private void placeBuildings(float p) {

        int[][] buildingGroups = new int[mapSize.width][mapSize.height];
        this.placedBuildings = 0;
        int placedBuildingsLast = 0;
        int groupNumber = 0, c = 0, maxBuildings = (int) (mapSize.width * mapSize.height * p);

        while (this.placedBuildings <= maxBuildings) {
            placedBuildingsLast = this.placedBuildings;
            try {
                Point pp = allPlaces.get(
                        (int) (Math.random() * allPlaces.size())
                );
                if (MAP[pp.x][pp.y] == EMPTY) {
                    allPlaces.remove(pp);
                    groupNumber++;
                    placeBuilding(pp, buildingGroups, groupNumber, false, 0, 2, maxBuildings);
                }
            } catch (Exception ex) {
            }
            if (placedBuildingsLast == this.placedBuildings) {
                c++;
            } else {
                c = 0;
            }
            if (c > 100) {
                return;
            }
        }
    }

    private void placeBuilding(
            final Point newPoint, final int[][] buildingGroups,
            final int groupNumber, boolean b, int depth, final int maxDepth,
            final int maxBuildings
    ) {
        if (maxBuildings < this.placedBuildings) {
            return;
        }
        if (depth > maxDepth) {
            return;
        }
        if (MAP[newPoint.x][newPoint.y] != EMPTY) {
            return;
        }
        for (int k = -1; k <= 1; k++) {
            for (int l = -1; l <= 1; l++) {

                if (Math.abs(k) == 1 || Math.abs(l) == 1) {
                    try {
                        if (MAP[newPoint.x + k][newPoint.y + l] != EMPTY) {
                            if (groupNumber != buildingGroups[newPoint.x + k][newPoint.y + l]
                                    || roadsAroundBuilding(new Point(newPoint.x + k, newPoint.x + l), newPoint) == 0) {
                                return;
                            }
                        }
                    } catch (Exception ex) {
                    }
                }
            }
        }
        try {
            if (b && (newPoint.x == 0 || newPoint.x == mapSize.width - 1)) {
                return;
            }
            if (b && (newPoint.y == 0 || newPoint.y == mapSize.height - 1)) {
                return;
            }
            if (newPoint.x == 0 || newPoint.x == mapSize.width - 1) {
                b = true;
            }
            if (newPoint.y == 0 || newPoint.y == mapSize.height - 1) {
                b = true;
            }
            buildingGroups[newPoint.x][newPoint.y] = groupNumber;
            MAP[newPoint.x][newPoint.y] = BUILDING;
            this.placedBuildings++;
            int i = (int) rnd.nextInt(2) + 1;
            for (int j = 0; j < i; j++) {
                Point pn = new Point(
                        (int) (newPoint.x + rnd.nextInt(2) - 1), (int) (newPoint.y + rnd.nextInt(2) - 1));
                placeBuilding(pn, buildingGroups, groupNumber, b, depth + 1, maxDepth, maxBuildings);
            }
        } catch (Exception ex) {
        }
    }

    private int roadsAroundBuilding(Point p, Point newP) {
        int n = 0;
        MAP[newP.x][newP.y] = BUILDING;
        for (int k = -1; k <= 1; k++) {
            for (int l = -1; l <= 1; l++) {
                if ((Math.abs(k) == 1 || Math.abs(l) == 1) && (Math.abs(k + l) != 2 && Math.abs(k + l) != 0)) {
                    try {
                        if (MAP[p.x + k][p.y + l] == EMPTY) {
                            n++;
                        }
                    } catch (Exception ex) {
                    }
                }
            }
        }
        MAP[newP.x][newP.y] = EMPTY;
        return n;
    }

    private void roadPlacing() {
        for (int x = 0; x < mapSize.width; x++) {
            for (int y = 0; y < mapSize.height; y++) {
                if (MAP[x][y] == EMPTY) {
                    boolean left = false, right = false, up = false, down = false;

                    try {
                        if (MAP[x - 1][y] != BUILDING) {
                            left = true;
                        }
                    } catch (Exception ex) {
                    }
                    try {
                        if (MAP[x + 1][y] != BUILDING) {
                            right = true;
                        }
                    } catch (Exception ex) {
                    }
                    try {
                        if (MAP[x][y - 1] != BUILDING) {
                            up = true;
                        }
                    } catch (Exception ex) {
                    }
                    try {
                        if (MAP[x][y + 1] != BUILDING) {
                            down = true;
                        }
                    } catch (Exception ex) {
                    }

                    if (!left && !right && !up && down) {
                        MAP[x][y] = UP_END;
                    } else if (!left && !right && up && !down) {
                        MAP[x][y] = DOWN_END;
                    } else if (!left && !right && up && down) {
                        MAP[x][y] = VERTICAL_STRAIGHT;
                    } else if (!left && right && !up && !down) {
                        MAP[x][y] = LEFT_END;
                    } else if (!left && right && !up && down) {
                        MAP[x][y] = FROM_DOWN_TO_RIGHT;
                    } else if (!left && right && up && !down) {
                        MAP[x][y] = FROM_UP_TO_RIGHT;
                    } else if (!left && right && up && down) {
                        MAP[x][y] = VERTICAL_STRAIGHT_AND_RIGHT;
                    } else if (left && !right && !up && !down) {
                        MAP[x][y] = RIGHT_END;
                    } else if (left && !right && !up && down) {
                        MAP[x][y] = FROM_DOWN_TO_LEFT;
                    } else if (left && !right && up && !down) {
                        MAP[x][y] = FROM_UP_TO_LEFT;
                    } else if (left && !right && up && down) {
                        MAP[x][y] = VERTICAL_STRAIGHT_AND_LEFT;
                    } else if (left && right && !up && !down) {
                        MAP[x][y] = HORIZONTAL_STRAIGHT;
                    } else if (left && right && !up && down) {
                        MAP[x][y] = HORIZONTAL_STRAIGHT_AND_DOWN;
                    } else if (left && right && up && !down) {
                        MAP[x][y] = HORIZONTAL_STRAIGHT_AND_UP;
                    } else if (left && right && up && down) {
                        MAP[x][y] = CROSSROAD;
                    }

                }
            }
        }
    }

    private void subCorrection1(int x, int y) {
        try {
            if (MAP[x][y] == HORIZONTAL_STRAIGHT_AND_DOWN
                    && MAP[x][y + 1] == HORIZONTAL_STRAIGHT_AND_UP) {
                MAP[x][y] = HORIZONTAL_STRAIGHT;
                MAP[x][y + 1] = HORIZONTAL_STRAIGHT;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == VERTICAL_STRAIGHT_AND_RIGHT
                    && MAP[x + 1][y] == VERTICAL_STRAIGHT_AND_LEFT) {
                MAP[x][y] = VERTICAL_STRAIGHT;
                MAP[x + 1][y] = VERTICAL_STRAIGHT;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == CROSSROAD
                    && MAP[x + 1][y] == VERTICAL_STRAIGHT_AND_LEFT) {
                MAP[x][y] = VERTICAL_STRAIGHT_AND_LEFT;
                MAP[x + 1][y] = VERTICAL_STRAIGHT;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == VERTICAL_STRAIGHT_AND_RIGHT
                    && MAP[x + 1][y] == CROSSROAD) {
                MAP[x][y] = VERTICAL_STRAIGHT;
                MAP[x + 1][y] = VERTICAL_STRAIGHT_AND_RIGHT;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == HORIZONTAL_STRAIGHT_AND_DOWN
                    && MAP[x][y + 1] == CROSSROAD) {
                MAP[x][y] = HORIZONTAL_STRAIGHT;
                MAP[x][y + 1] = HORIZONTAL_STRAIGHT_AND_DOWN;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == CROSSROAD
                    && MAP[x][y + 1] == HORIZONTAL_STRAIGHT_AND_UP) {
                MAP[x][y] = HORIZONTAL_STRAIGHT_AND_UP;
                MAP[x][y + 1] = HORIZONTAL_STRAIGHT;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == CROSSROAD
                    && MAP[x + 1][y] == CROSSROAD) {
                MAP[x][y] = VERTICAL_STRAIGHT_AND_LEFT;
                MAP[x + 1][y] = VERTICAL_STRAIGHT_AND_RIGHT;
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == CROSSROAD
                    && MAP[x][y + 1] == CROSSROAD) {
                MAP[x][y] = HORIZONTAL_STRAIGHT_AND_UP;
                MAP[x][y + 1] = HORIZONTAL_STRAIGHT_AND_DOWN;
            }
        } catch (Exception ex) {
        }
    }

    private void subCorrection2(int x, int y) {
        try {
            if (MAP[x][y] == HORIZONTAL_STRAIGHT
                    && MAP[x][y + 1] == HORIZONTAL_STRAIGHT
                    && MAP[x][y + 2] == HORIZONTAL_STRAIGHT) {
                MAP[x][y + 1] = BUILDING;
                try {
                    switch (MAP[x + 1][y + 1]) {
                        case VERTICAL_STRAIGHT_AND_LEFT:
                            MAP[x + 1][y + 1] = VERTICAL_STRAIGHT;
                            break;
                        case HORIZONTAL_STRAIGHT_AND_DOWN:
                            MAP[x + 1][y + 1] = FROM_DOWN_TO_RIGHT;
                            break;
                        case HORIZONTAL_STRAIGHT_AND_UP:
                            MAP[x + 1][y + 1] = FROM_UP_TO_RIGHT;
                            break;
                        case HORIZONTAL_STRAIGHT:
                            MAP[x + 1][y + 1] = LEFT_END;
                            break;
                        case FROM_UP_TO_LEFT:
                            MAP[x + 1][y + 1] = DOWN_END;
                            break;
                        case FROM_DOWN_TO_LEFT:
                            MAP[x + 1][y + 1] = UP_END;
                            break;
                        case CROSSROAD:
                            MAP[x + 1][y + 1] = VERTICAL_STRAIGHT_AND_RIGHT;
                            break;
                        default:
                            break;
                    }
                } catch (Exception ex) {
                }
                try {
                    switch (MAP[x - 1][y + 1]) {
                        case VERTICAL_STRAIGHT_AND_RIGHT:
                            MAP[x - 1][y + 1] = VERTICAL_STRAIGHT;
                            break;
                        case HORIZONTAL_STRAIGHT_AND_DOWN:
                            MAP[x - 1][y + 1] = FROM_DOWN_TO_LEFT;
                            break;
                        case HORIZONTAL_STRAIGHT_AND_UP:
                            MAP[x - 1][y + 1] = FROM_UP_TO_LEFT;
                            break;
                        case HORIZONTAL_STRAIGHT:
                            MAP[x - 1][y + 1] = RIGHT_END;
                            break;
                        case FROM_UP_TO_LEFT:
                            MAP[x - 1][y + 1] = DOWN_END;
                            break;
                        case FROM_DOWN_TO_LEFT:
                            MAP[x - 1][y + 1] = UP_END;
                            break;
                        case CROSSROAD:
                            MAP[x - 1][y + 1] = VERTICAL_STRAIGHT_AND_LEFT;
                            break;
                        default:
                            break;
                    }
                } catch (Exception ex) {
                }
                return;
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[x][y] == VERTICAL_STRAIGHT
                    && MAP[x + 1][y] == VERTICAL_STRAIGHT
                    && MAP[x + 2][y] == VERTICAL_STRAIGHT) {
                MAP[x + 1][y] = BUILDING;
                try {
                    switch (MAP[x + 1][y + 1]) {
                        case HORIZONTAL_STRAIGHT_AND_UP:
                            MAP[x + 1][y + 1] = HORIZONTAL_STRAIGHT;
                            break;
                        case VERTICAL_STRAIGHT_AND_RIGHT:
                            MAP[x + 1][y + 1] = FROM_DOWN_TO_RIGHT;
                            break;
                        case VERTICAL_STRAIGHT_AND_LEFT:
                            MAP[x + 1][y + 1] = FROM_DOWN_TO_LEFT;
                            break;
                        case VERTICAL_STRAIGHT:
                            MAP[x + 1][y + 1] = UP_END;
                            break;
                        case FROM_UP_TO_LEFT:
                            MAP[x + 1][y + 1] = RIGHT_END;
                            break;
                        case FROM_UP_TO_RIGHT:
                            MAP[x + 1][y + 1] = LEFT_END;
                            break;
                        case CROSSROAD:
                            MAP[x + 1][y + 1] = HORIZONTAL_STRAIGHT_AND_DOWN;
                            break;
                        default:
                            break;
                    }
                } catch (Exception ex) {
                }
                try {
                    switch (MAP[x + 1][y - 1]) {
                        case HORIZONTAL_STRAIGHT_AND_DOWN:
                            MAP[x + 1][y - 1] = HORIZONTAL_STRAIGHT;
                            break;
                        case VERTICAL_STRAIGHT_AND_RIGHT:
                            MAP[x + 1][y - 1] = FROM_UP_TO_RIGHT;
                            break;
                        case VERTICAL_STRAIGHT_AND_LEFT:
                            MAP[x + 1][y - 1] = FROM_UP_TO_LEFT;
                            break;
                        case VERTICAL_STRAIGHT:
                            MAP[x + 1][y - 1] = DOWN_END;
                            break;
                        case FROM_DOWN_TO_LEFT:
                            MAP[x + 1][y - 1] = RIGHT_END;
                            break;
                        case FROM_DOWN_TO_RIGHT:
                            MAP[x + 1][y - 1] = LEFT_END;
                            break;
                        case CROSSROAD:
                            MAP[x + 1][y - 1] = HORIZONTAL_STRAIGHT_AND_UP;
                            break;
                        default:
                            break;
                    }
                } catch (Exception ex) {
                }
            }
        } catch (Exception ex) {
        }
    }

    /**
     * Generate Data
     * ###################################################################
     */
    public String generateData() {
        if (MAP == null) {
            return "null";
        }
        ArrayList<Point> points = new ArrayList<>();
        ArrayList<String> roads = new ArrayList<>();

        String data = "";
        int c = 0;
        for (int x = 0; x < mapSize.width; x++) {
            for (int y = 0; y < mapSize.height; y++) {
                if (MAP[x][y] != VERTICAL_STRAIGHT
                        && MAP[x][y] != HORIZONTAL_STRAIGHT
                        && MAP[x][y] != FROM_DOWN_TO_LEFT
                        && MAP[x][y] != FROM_DOWN_TO_RIGHT
                        && MAP[x][y] != FROM_UP_TO_LEFT
                        && MAP[x][y] != FROM_UP_TO_RIGHT
                        && MAP[x][y] != BUILDING
                        && MAP[x][y] != EMPTY) {
                    data += "[" + x + ";" + y + "]" + "/";
                    points.add(new Point(x, y));
                    c++;
                }
            }
        }
        data = c + "/" + data;

        try {
            for (int i = 0; i < points.size(); i++) {
                Point p = new Point(points.get(i).x, points.get(i).y);
                RoadTracing rT = new RoadTracing(p);
                calcRoadDistance(rT, points, roads, i - 1);
            }
            data += roads.size() + "/";
            for (String s : roads) {
                data += s + "/";
            }
        } catch (Exception ex) {
        }

        return data;
    }

    private void calcRoadDistance(RoadTracing roadTracing, final ArrayList<Point> points, final ArrayList<String> roads, final int last) {
        Point p = roadTracing.position;

        for (int i = 0; i < points.size(); i++) {
            if (points.get(i).equals(p) && !p.equals(roadTracing.getStart())) {
                if (i <= last) {
                    return;
                }
                int a = 0;
                for (int j = 0; j < points.size(); j++) {
                    if (points.get(j).equals(roadTracing.getStart())) {
                        a = j + 1;
                    }
                }
                int b = i + 1;
                roads.add(a + "(" + roadTracing.length + ")" + b);
            }
        }
        try {
            if (MAP[p.x + 1][p.y] != BUILDING && roadTest(roadTracing.RIGHT, MAP[p.x + 1][p.y])) {
                RoadTracing r = cloneRoadPath(roadTracing);
                r.next(r.RIGHT);
                boolean b = true;
                for (Point pp : r.lastPositions) {
                    if (pp.equals(r.position)) {
                        b = false;
                    }
                }
                if (b) {
                    r.addPosition();
                    calcRoadDistance(r, points, roads, last);
                }
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[p.x - 1][p.y] != BUILDING && roadTest(roadTracing.LEFT, MAP[p.x - 1][p.y])) {
                RoadTracing r = cloneRoadPath(roadTracing);
                r.next(r.LEFT);
                boolean b = true;
                for (Point pp : r.lastPositions) {
                    if (pp.equals(r.position)) {
                        b = false;
                    }
                }
                if (b) {
                    r.addPosition();
                    calcRoadDistance(r, points, roads, last);
                }
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[p.x][p.y + 1] != BUILDING && roadTest(roadTracing.DOWN, MAP[p.x][p.y + 1])) {
                RoadTracing r = cloneRoadPath(roadTracing);
                r.next(r.DOWN);
                boolean b = true;
                for (Point pp : r.lastPositions) {
                    if (pp.equals(r.position)) {
                        b = false;
                    }
                }
                if (b) {
                    r.addPosition();
                    calcRoadDistance(r, points, roads, last);
                }
            }
        } catch (Exception ex) {
        }
        try {
            if (MAP[p.x][p.y - 1] != BUILDING && roadTest(roadTracing.UP, MAP[p.x][p.y - 1])) {
                RoadTracing r = cloneRoadPath(roadTracing);
                r.next(r.UP);
                boolean b = true;
                for (Point pp : r.lastPositions) {
                    if (pp.equals(r.position)) {
                        b = false;
                    }
                }
                if (b) {
                    r.addPosition();
                    calcRoadDistance(r, points, roads, last);
                }
            }
        } catch (Exception ex) {
        }
    }

    private RoadTracing cloneRoadPath(RoadTracing RoadTracing) {
        RoadTracing r = new RoadTracing(RoadTracing.position);
        r.lastPositions = (ArrayList<Point>) RoadTracing.lastPositions.clone();
        r.length = RoadTracing.length;
        r.startPosition = RoadTracing.startPosition;
        return r;
    }

    private boolean roadTest(int d, int road) {
        if (d == 0 && (road == CROSSROAD
                || road == FROM_DOWN_TO_RIGHT
                || road == FROM_UP_TO_RIGHT
                || road == HORIZONTAL_STRAIGHT
                || road == LEFT_END
                || road == VERTICAL_STRAIGHT_AND_RIGHT
                || road == HORIZONTAL_STRAIGHT_AND_DOWN
                || road == HORIZONTAL_STRAIGHT_AND_UP)) {
            return true;
        }
        if (d == 1 && (road == CROSSROAD
                || road == FROM_DOWN_TO_LEFT
                || road == FROM_DOWN_TO_RIGHT
                || road == VERTICAL_STRAIGHT
                || road == UP_END
                || road == HORIZONTAL_STRAIGHT_AND_DOWN
                || road == VERTICAL_STRAIGHT_AND_LEFT
                || road == VERTICAL_STRAIGHT_AND_RIGHT)) {
            return true;
        }
        if (d == 2 && (road == CROSSROAD
                || road == FROM_DOWN_TO_LEFT
                || road == FROM_UP_TO_LEFT
                || road == HORIZONTAL_STRAIGHT
                || road == RIGHT_END
                || road == VERTICAL_STRAIGHT_AND_LEFT
                || road == HORIZONTAL_STRAIGHT_AND_DOWN
                || road == HORIZONTAL_STRAIGHT_AND_UP)) {
            return true;
        }
        if (d == 3 && (road == CROSSROAD
                || road == FROM_UP_TO_LEFT
                || road == FROM_UP_TO_RIGHT
                || road == VERTICAL_STRAIGHT
                || road == DOWN_END
                || road == HORIZONTAL_STRAIGHT_AND_UP
                || road == VERTICAL_STRAIGHT_AND_LEFT
                || road == VERTICAL_STRAIGHT_AND_RIGHT)) {
            return true;
        }
        return false;
    }

}
