/***********************************************************************************************/
/*                                                                                             */
/*										 = T O W N G E N =	    							   */
/*											 verze 1.0										   */
/*                                        Generator mesta                                      */
/*																							   */
/*											 DARK CODER										   */
/*											    2017										   */
/*																							   */
/***********************************************************************************************/

#define _CRT_SECURE_NO_WARNINGS

// Konstanty
#define DEBUG 0 // Debug mod

#define UNKN 0 // nezname
#define BUIL 1 // budova
#define RO12 2 // zatacka vpravo vpred
#define RO23 3 // zatacka vlevo vpred
#define RO14 4 // zatacka vpravo zpet
#define RO34 5 // zatacka vlevo zpet
#define RO24 6 // cesta svisla
#define RO13 7 // cesta vodorovna

#define TILESIRKA 64 // sirka tilesu
#define TILEVYSKA 64 // vyska tilesu

// Knihovny
#include <SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Prototypy
void inicialization(char *filename);
void generate_map(int sirka, int vyska);
void create_image_file(char *filename);
void create_data_file(char *filename);
void deinicialization(void);

// Struktura pro tilesy
struct sTile {
	char file[13]; // jmeno souboru tilesu
	SDL_Rect rect; // rozmery tilesu
	SDL_Surface *surface; // surface tilesu
};

// Globalni promenne
char mesto_jmeno[21]; // nazev mesta
char map_jmeno[25]; // jmeno mapoveho souboru
char dat_jmeno[25]; // jmeno datoveho souboru
int sirka; // velikost mesta na sirku
int vyska; // velikost mesta na vysku
int i, j; // countery
int kks; // koeficient kratsi strany (sirka vs vyska mesta)
int ksl; // koeficient sudosti a lichosti (1 - licha, 2 - suda)
int *mesto2D; // dynamicke 2D pole reprezentujici mesto
int init; // status inicializace SDL
struct sTile image[8]; // Tilesy

int main(int argc, char * argv[]){
	
	// Kontrola spravneho poctu argumentu 
	if (argc != 4) {
		printf("Chybny pocet zadanych parametru\n");
		printf("Pouziti: towngen.exe <nazev> <sirka> <vyska>\n");
		exit(1);
	}

	// Kontrola spravnosti nazvu aplikace
	if (strcmp(argv[0], "towngen.exe")) {
		printf("Chybny nazev aplikace\n");
		exit(1);
	}

	// Kontrola delky nazvu mesta
	if (strlen(argv[1]) > 20){
		printf("Priliz dlouhy nazev mesta\n");
		printf("Delka nazvu mesta je omezena na 20 znaku\n");
		exit(1);
	}
	else {
		strcpy(mesto_jmeno, argv[1]);
		strcpy(map_jmeno, mesto_jmeno);
		strcat(map_jmeno, ".bmp");
		strcpy(dat_jmeno, mesto_jmeno);
		strcat(dat_jmeno, ".dat");
	}

	// Kontrola spravnosti parametru sirka
	sirka = atoi(argv[2]);
	if ((sirka < 1)||(sirka > 99)) {
		printf("Chybny parametr sirka\n");
		printf("Zadejte cislo v rozsahu 1-99\n");
		exit(1);
	}

	// Kontrola spravnosti parametru vyska
	vyska = atoi(argv[3]);
	if ((vyska < 1) || (vyska > 99)) {
		printf("Chybny parametr vyska\n");
		printf("Zadejte cislo v rozsahu 1-99\n");
		exit(1);
	}

	// Inicializace SDL
	init = SDL_Init(SDL_INIT_VIDEO);
	if (init) {
		printf("Chyba inicializace SDL\n");
		exit(1);
	}

	inicialization("towngen.ini");
	
	generate_map(sirka, vyska);

	create_image_file(map_jmeno);

	create_data_file(dat_jmeno);

	deinicialization();
	
	return 0;
}

// Nacteni dat z inicializacniho souboru
void inicialization(char *filename) {
	FILE *fr;

	// Otevreni textoveho souboru pro cteni
	fr = fopen(filename, "r");
	if (fr == NULL) {
		printf("Chyba pri otevirani souboru %s\n", filename);
		exit(1);
	}

	// Nacteni nazvu souboru tilesu
	fscanf(fr, "%s", image[UNKN].file);
	fscanf(fr, "%s", image[BUIL].file);
	fscanf(fr, "%s", image[RO12].file);
	fscanf(fr, "%s", image[RO23].file);
	fscanf(fr, "%s", image[RO14].file);
	fscanf(fr, "%s", image[RO34].file);
	fscanf(fr, "%s", image[RO24].file);
	fscanf(fr, "%s", image[RO13].file);

	// Uzavreni souboru
	if (fclose(fr)==EOF) {
		printf("Chyba pri uzavirani souboru %s\n", filename);
		exit(1);
	}

	// Nacteni tilesu do pameti
	image[UNKN].surface = SDL_LoadBMP(image[UNKN].file);
	image[UNKN].rect.w = image[UNKN].surface->w;
	image[UNKN].rect.h = image[UNKN].surface->h;

	image[BUIL].surface = SDL_LoadBMP(image[BUIL].file);
	image[BUIL].rect.w = image[BUIL].surface->w;
	image[BUIL].rect.h = image[BUIL].surface->h;

	image[RO12].surface = SDL_LoadBMP(image[RO12].file);
	image[RO12].rect.w = image[RO12].surface->w;
	image[RO12].rect.h = image[RO12].surface->h;

	image[RO23].surface = SDL_LoadBMP(image[RO23].file);
	image[RO23].rect.w = image[RO23].surface->w;
	image[RO23].rect.h = image[RO23].surface->h;

	image[RO14].surface = SDL_LoadBMP(image[RO14].file);
	image[RO14].rect.w = image[RO14].surface->w;
	image[RO14].rect.h = image[RO14].surface->h;

	image[RO34].surface = SDL_LoadBMP(image[RO34].file);
	image[RO34].rect.w = image[RO34].surface->w;
	image[RO34].rect.h = image[RO34].surface->h;

	image[RO24].surface = SDL_LoadBMP(image[RO24].file);
	image[RO24].rect.w = image[RO24].surface->w;
	image[RO24].rect.h = image[RO24].surface->h;

	image[RO13].surface = SDL_LoadBMP(image[RO13].file);
	image[RO13].rect.w = image[RO13].surface->w;
	image[RO13].rect.h = image[RO13].surface->h;
}

// Generovani mesta
void generate_map(int sirka, int vyska) {

	// Kontrola vytvoreni spojite cesty a rozmisteni budov
	if (((sirka < 2) || (vyska < 2)) || ((sirka == 2) && (vyska == 2))) {
		printf("Nelze vytvorit spojitou cestu a rozmistit budovy\n");
		printf("Mesto nebylo vygenerovano\n");
		exit(1);
	}

	// Alokace pameti pro 2D pole
	mesto2D = (int *)malloc(sirka * vyska * sizeof(int));
	if (!mesto2D) {
		printf("Chyba alokace pameti");
		exit(1);
	}

	// Reset 2D pole
	for (i = 0; i < vyska; i++) {
		for (j = 0; j < sirka; j++) {
			mesto2D[i * sirka + j] = UNKN;
		}
	}

	// Prizpusobeni vypoctu dle orientace a sudosti nebo lichosti
	if (sirka < vyska) { // orientace na vysku
		kks = sirka;
		if (vyska % 2) { // licha
			ksl = 1;
			// Budova
			for (j = 0; j<sirka; j++) mesto2D[0 * sirka + j] = BUIL;
			// Zatacka vpravo vpred
			mesto2D[1 * sirka + 0] = RO12;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 2; i < (vyska - 1); i += 2) mesto2D[i * sirka + 1] = RO12;
			}
			// Zatacka vlevo vpred
			mesto2D[1 * sirka + (sirka - 1)] = RO23;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 3; i < (vyska - 1); i += 2) mesto2D[i * sirka + (sirka - 1)] = RO23;
			}
			// Zatacka vpravo zpet
			mesto2D[(vyska - 1) * sirka + 0] = RO14;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 3; i < (vyska - 1); i += 2) mesto2D[i * sirka + 1] = RO14;
			}
			// Zatacka vlevo zpet
			mesto2D[(vyska - 1) * sirka + (sirka - 1)] = RO34;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 2; i < (vyska - 1); i += 2) mesto2D[i * sirka + (sirka - 1)] = RO34;
			}
			// Svisla cesta
			if (vyska > 4) {
				for (i = 2; i < (vyska - 1); i++) mesto2D[i * sirka + 0] = RO24;
				if (sirka < 3) {
					for (i = 2; i < (vyska - 1); i++) mesto2D[i * sirka + (sirka - 1)] = RO24;
				}
			}
			// Vodorovna cesta
			if (sirka > 2) {
				for (j = 1; j < (sirka - 1); j++) mesto2D[1 * sirka + j] = RO13;
				for (j = 1; j < (sirka - 1); j++) mesto2D[(vyska - 1) * sirka + j] = RO13;
			}
			if ((sirka > 3) && (vyska > 4)) {
				for (j = 2; j < (sirka - 1); j++) {
					for (i = 2; i < (vyska - 1); i++) mesto2D[i * sirka + j] = RO13;
				}
			}
		}
		else { // suda
			ksl = 2;
			// Budova
			for (j = 0; j<sirka; j++) mesto2D[0 * sirka + j] = BUIL;
			for (j = 0; j<sirka; j++) mesto2D[(vyska - 1) * sirka + j] = BUIL;
			// Zatacka vpravo vpred
			mesto2D[1 * sirka + 0] = RO12;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 2; i < (vyska - 2); i += 2) mesto2D[i * sirka + 1] = RO12;
			}
			// Zatacka vlevo vpred
			mesto2D[1 * sirka + (sirka - 1)] = RO23;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 3; i < (vyska - 2); i += 2) mesto2D[i * sirka + (sirka - 1)] = RO23;
			}
			// Zatacka vpravo zpet
			mesto2D[(vyska - 2) * sirka + 0] = RO14;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 3; i < (vyska - 2); i += 2) mesto2D[i * sirka + 1] = RO14;
			}
			// Zatacka vlevo zpet
			mesto2D[(vyska - 2) * sirka + (sirka - 1)] = RO34;
			if ((sirka > 2) && (vyska > 4)) {
				for (i = 2; i < (vyska - 2); i += 2) mesto2D[i * sirka + (sirka - 1)] = RO34;
			}
			// Svisla cesta
			if (vyska > 4) {
				for (i = 2; i < (vyska - 2); i++) mesto2D[i * sirka + 0] = RO24;
				if (sirka < 3) {
					for (i = 2; i < (vyska - 2); i++) mesto2D[i * sirka + (sirka - 1)] = RO24;
				}
			}
			// Vodorovna cesta
			if (sirka > 2) {
				for (j = 1; j < (sirka - 1); j++) mesto2D[1 * sirka + j] = RO13;
				for (j = 1; j < (sirka - 1); j++) mesto2D[(vyska - 2) * sirka + j] = RO13;
			}
			if ((sirka > 3) && (vyska > 4)) {
				for (j = 2; j < (sirka - 1); j++) {
					for (i = 2; i < (vyska - 2); i++) mesto2D[i * sirka + j] = RO13;
				}
			}
		}
	}
	else { 
		kks = vyska;
		if (sirka % 2) { // licha
			ksl = 1;
			// Budova
			for (i = 0; i<vyska; i++) mesto2D[i * sirka + (sirka - 1)] = BUIL;
			// Zatacka vpravo vpred
			mesto2D[0 * sirka + 0] = RO12;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 1; j < (sirka - 2); j += 2) mesto2D[1 * sirka + j] = RO12;
			}
			// Zatacka vlevo vpred
			mesto2D[0 * sirka + (sirka - 2)] = RO23;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 2; j < (sirka - 2); j += 2) mesto2D[1 * sirka + j] = RO23;
			}
			// Zatacka vpravo zpet
			mesto2D[(vyska - 1) * sirka + 0] = RO14;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 2; j < (sirka - 2); j += 2) mesto2D[(vyska - 1) * sirka + j] = RO14;
			}
			// Zatacka vlevo zpet
			mesto2D[(vyska - 1) * sirka + (sirka - 2)] = RO34;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 1; j < (sirka - 2); j += 2) mesto2D[(vyska - 1) * sirka + j] = RO34;
			}
			// Svisla cesta
			if (vyska > 2) {
				for (i = 1; i < (vyska - 1); i++) mesto2D[i * sirka + 0] = RO24;
				for (i = 1; i < (vyska - 1); i++) mesto2D[i * sirka + (sirka - 2)] = RO24;
			}
			if ((sirka > 4) && (vyska > 3)) {
				for (i = 2; i < (vyska - 1); i++) {
					for (j = 1; j < (sirka - 2); j++) mesto2D[i * sirka + j] = RO24;
				}
			}
			// Vodorovna cesta
			if (sirka > 4) {
				for (j = 1; j < (sirka - 2); j++) mesto2D[0 * sirka + j] = RO13;
				if (vyska < 3) {
					for (j = 1; j < (sirka - 2); j++) mesto2D[(vyska - 1) * sirka + j] = RO13;
				}
			}
		}
		else { // suda
			ksl = 2;
			// Budova
			for (i = 0; i<vyska; i++) mesto2D[i * sirka + (sirka - 1)] = BUIL;
			for (i = 0; i<vyska; i++) mesto2D[i * sirka + 0] = BUIL;
			// Zatacka vpravo vpred
			mesto2D[0 * sirka + 1] = RO12;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 2; j < (sirka - 2); j += 2) mesto2D[1 * sirka + j] = RO12;
			}
			// Zatacka vlevo vpred
			mesto2D[0 * sirka + (sirka - 2)] = RO23;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 3; j < (sirka - 2); j += 2) mesto2D[1 * sirka + j] = RO23;
			}
			// Zatacka vpravo zpet
			mesto2D[(vyska - 1) * sirka + 1] = RO14;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 3; j < (sirka - 2); j += 2) mesto2D[(vyska - 1) * sirka + j] = RO14;
			}
			// Zatacka vlevo zpet
			mesto2D[(vyska - 1) * sirka + (sirka - 2)] = RO34;
			if ((sirka > 4) && (vyska > 2)) {
				for (j = 2; j < (sirka - 2); j += 2) mesto2D[(vyska - 1) * sirka + j] = RO34;
			}
			// Svisla cesta
			if (vyska > 2) {
				for (i = 1; i < (vyska - 1); i++) mesto2D[i * sirka + 1] = RO24;
				for (i = 1; i < (vyska - 1); i++) mesto2D[i * sirka + (sirka - 2)] = RO24;
			}
			if ((sirka > 4) && (vyska > 3)) {
				for (i = 2; i < (vyska - 1); i++) {
					for (j = 2; j < (sirka - 2); j++) mesto2D[i * sirka + j] = RO24;
				}
			}
			// Vodorovna cesta
			if (sirka > 4) {
				for (j = 2; j < (sirka - 2); j++) mesto2D[0 * sirka + j] = RO13;
				if (vyska < 3) {
					for (j = 2; j < (sirka - 2); j++) mesto2D[(vyska - 1) * sirka + j] = RO13;
				}
			}
		}
	}
}

// Vytvoreni obrazoveho souboru mesta 
void create_image_file(char *filename) {
	SDL_Surface* map_surface;
	Uint32 rmask, gmask, bmask, amask;
	SDL_Rect temp;

	// Nastaveni poradi vykreslovani barev
    #if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
	#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
	#endif

	// Vytvoreni prazdneho surface
	map_surface = SDL_CreateRGBSurface(0, sirka * TILESIRKA, vyska * TILEVYSKA, 24, rmask, gmask, bmask, amask);
	if (map_surface == NULL) exit(1);

	// Zakreslovani tilesu do surface
	for (i = 0; i < vyska; i++) {
		for (j = 0; j < sirka; j++) {
			temp.x = j * TILESIRKA;
			temp.y = i * TILEVYSKA;
			SDL_BlitSurface(image[mesto2D[i * sirka + j]].surface, NULL, map_surface, &temp);
		}
	}
	
	// Ulozeni surface do souboru
	SDL_SaveBMP(map_surface, filename);

	// Uvolneni pameti surface
	SDL_FreeSurface(map_surface);
}

// Vytvoreni datoveho souboru a ulozeni dat
void create_data_file(char *filename) {

	FILE *fw;

	// Otevreni textoveho souboru pro zapis
	fw = fopen(filename, "w");
	if (fw == NULL) {
		printf("Chyba pri otevirani souboru %s\n", filename);
		exit(1);
	}

	// Zapis dat
	fprintf(fw, "Nazev mesta: %s\n", mesto_jmeno);
	fprintf(fw, "Rozloha mesta: %d (%dx%d)\n", sirka * vyska, sirka, vyska);
	fprintf(fw, "Pocet budov: %d\n", ksl * kks);
	fprintf(fw, "Delka cesty: %d\n", sirka * vyska - (ksl * kks));
	fprintf(fw, "Soubor map: %s\n", map_jmeno);
	fprintf(fw, "Graf neobsahuje vrcholy\n\n");

	for (i = 0; i < vyska; i++) {
		for (j = 0; j < sirka; j++) {
			fprintf(fw, "%d ", mesto2D[i * sirka + j]);
			if (j == (sirka - 1)) fprintf(fw, "\n");
		}
	}

	fprintf(fw, "\n");
	fprintf(fw, "Vysvetlivky\n");
	fprintf(fw, "1 - budova\n");
	fprintf(fw, "2,3,4,5,6,7 - cesta\n\n");

	// Uzavreni souboru
	if (fclose(fw) == EOF) {
		printf("Chyba pri uzavirani souboru %s\n", filename);
		exit(1);
	}
}

// Deinicializace a uvolneni pameti
void deinicialization(void) {
	int i;

	free(mesto2D);
	for (i = 0; i < 8; i++) SDL_FreeSurface(image[i].surface);
	SDL_Quit();
}