﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TownGenerator.TownGeneration;
using static TownGenerator.Town;

namespace TownGenerator
{
    public sealed class Crossroad : Node
    {
        private Tile[,] tiles;

        public Crossroad(Node node, Tile[,] tiles, Crossroad lastCrossroad, int distance)
            : base(node.X, node.Y)
        {
            this.tiles = tiles;
            LastCrossroad = lastCrossroad;
            Distance = distance;
        }

        public bool CanGoRight { get; set; }
        public bool CanGoUp { get; set; }
        public bool CanGoLeft { get; set; }
        public bool CanGoDown { get; set; }

        public int Distance { get; }

        public Crossroad LastCrossroad { get; }

        public Direction GetDirection()
        {
            if (CanGoRight) return Direction.Right;
            if (CanGoUp) return Direction.Up;
            if (CanGoLeft) return Direction.Left;
            if (CanGoDown) return Direction.Down;

            return Direction.None;
        }

        private static bool CanTurnRight(Node node, Tile[,] parts, Direction lastDirection)
        {
            if (lastDirection != Direction.Left)
            {
                if (node.X + 1 < parts.GetLength(0))
                {
                    if (parts[node.X + 1, node.Y].TileType == TileType.Road)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        private static bool CanTurnUp(Node node, Tile[,] parts, Direction lastDirection)
        {
            if (lastDirection != Direction.Down)
            {
                if (node.Y - 1 >= 0)
                {
                    if (parts[node.X, node.Y - 1].TileType == TileType.Road)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        private static bool CanTurnLeft(Node node, Tile[,] tiles, Direction lastDirection)
        {
            if (lastDirection != Direction.Right)
            {
                if (node.X - 1 >= 0)
                {
                    if (tiles[node.X - 1, node.Y].TileType == TileType.Road)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        private static bool CanTurnDown(Node node, Tile[,] tiles, Direction lastDirection)
        {
            if (lastDirection != Direction.Up)
            {
                if (node.Y + 1 < tiles.GetLength(1))
                {
                    if (tiles[node.X, node.Y + 1].TileType == TileType.Road)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        public static bool TryGetCrossroad(Node node, Tile[,] tiles, Crossroad lastCrossroad,
        Direction last, int distance, out Crossroad newCrossroad)
        {
            bool canGoLeft = CanTurnLeft(node, tiles, last);
            bool canGoRight = CanTurnRight(node, tiles, last);
            bool canGoUp = CanTurnUp(node, tiles, last);
            bool canGoDown = CanTurnDown(node, tiles, last);

            if (IsCrossroad(canGoRight, canGoUp, canGoLeft, canGoDown))
            {
                newCrossroad = new Crossroad(node, tiles, lastCrossroad, distance)
                {
                    CanGoRight = canGoRight,
                    CanGoUp = canGoUp,
                    CanGoLeft = canGoLeft,
                    CanGoDown = canGoDown
                };
                return true;
            }

            newCrossroad = null;
            return false;
        }

        internal static Crossroad FromNode(Node node, Tile[,] parts)
        {
            return new Crossroad(node, parts, null, 0)
            {
                CanGoRight = CanTurnRight(node, parts, Direction.None),
                CanGoUp = CanTurnUp(node, parts, Direction.None),
                CanGoLeft = CanTurnLeft(node, parts, Direction.None),
                CanGoDown = CanTurnDown(node, parts, Direction.None)
            };
        }

        private static bool IsCrossroad(bool can1, bool can2, bool can3, bool can4)
        {
            int numberOfPossibleDirections = 0;
            if (can1) numberOfPossibleDirections++;
            if (can2) numberOfPossibleDirections++;
            if (can3) numberOfPossibleDirections++;
            if (can4) numberOfPossibleDirections++;

            return numberOfPossibleDirections > 1;
        }
    }
}
