﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TownGenerator.TownGeneration;

namespace TownGenerator
{
    public sealed class TownBuilder
    {
        public Town Build(int width, int height)
        {
            Tile[,] tiles = new Tile[width, height];

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    if (x == 0 || y == 0)
                        tiles[x, y] = new Tile(TileType.Road);
                    else
                    {
                        int i = x % 3;
                        int j = y % 3;
                        if ((i == 1 || i == 2) && (j == 1 || j == 2))
                            tiles[x, y] = new Tile(TileType.Building);
                        else
                            tiles[x, y] = new Tile(TileType.Road);
                    }
                }
            }

            // make sure, the building in the corner has road next to it
            if (width % 3 == 0 && height % 3 == 0)
            {
                tiles[width - 1, height - 1] = new Tile(TileType.Road);
                tiles[width - 2, height - 1] = new Tile(TileType.Road);
            }

            SetTilesNeighbours(tiles);

            List<Node> nodes = GetNodes(tiles);

            return new Town(tiles, nodes);
        }

        private void SetTilesNeighbours(Tile[,] tiles)
        {
            for (int y = 0; y < tiles.GetLength(1); y++)
            {
                for (int x = 0; x < tiles.GetLength(0); x++)
                {
                    if (x - 1 < 0) tiles[x, y].LeftTile = new Tile(TileType.None);
                    else tiles[x, y].LeftTile = tiles[x - 1, y];
                    if (y - 1 < 0) tiles[x, y].TopTile = new Tile(TileType.None);
                    else tiles[x, y].TopTile = tiles[x, y - 1];
                    if (x + 1 >= tiles.GetLength(0)) tiles[x, y].RightTile = new Tile(TileType.None);
                    else tiles[x, y].RightTile = tiles[x + 1, y];
                    if (y + 1 >= tiles.GetLength(1)) tiles[x, y].BottomTile = new Tile(TileType.None);
                    else tiles[x, y].BottomTile = tiles[x, y + 1];
                }
            }
        }

        private List<Node> GetNodes(Tile[,] tiles)
        {
            List<Node> nodes = new List<Node>();

            for (int x = 0; x < tiles.GetLength(0); x++)
            {
                for (int y = 0; y < tiles.GetLength(1); y++)
                {
                    if (x == 0 && y == 0)
                        continue;

                    if (tiles[x, y].TileType == TileType.Road)
                    {
                        int i = (x - 3) % 3;
                        int j = (y - 3) % 3;

                        // pokud je to krizovatka.. 
                        if (i == 0 && j == 0)
                        {
                            // vyloucime levy horni roh a pravy dolni roh
                            if (x > 0 && y > 0 && (x < tiles.GetLength(0) - 1
                                || y < tiles.GetLength(1) - 1))
                                nodes.Add(new Node(x, y));
                        }
                        if (y == 0 && i == 0)
                        {
                            if (x != tiles.GetLength(0) - 1)
                                nodes.Add(new Node(x, y));
                        }
                        if (x == 0 && j == 0)
                        {
                            if (y != tiles.GetLength(1) - 1)
                                nodes.Add(new Node(x, y));
                        }

                        // pokud je to konec cesty..
                        // pokud je to u spodní hrany mapy
                        if (y == (tiles.GetLength(1) - 1))
                        {
                            if (y % 3 == 1)
                                nodes.Add(new Node(x, y));
                            if (y % 3 == 2 && x < tiles.GetLength(0) - 3)
                                nodes.Add(new Node(x, y));
                        }
                        // pokud je to u pravé hrany mapy
                        if (x == (tiles.GetLength(0) - 1))
                        {
                            if (x % 3 == 1 || x % 3 == 2)
                            {
                                Node node = new Node(x, y);
                                if (!nodes.Contains(node))
                                    nodes.Add(new Node(x, y));
                            }
                        }
                        // pokud je to v rohu
                        //if (x == (parts.GetLength(0) - 1) && y == (parts.GetLength(1) - 1)
                        //    && x % 3 != 0 && y % 3 != 0)
                        //    nodes.Add(new Node(x, y));
                    }
                }
            }

            return nodes;
        }
    }
}
