﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TownGenerator.TownGeneration;

namespace TownGenerator
{
    public sealed class Writer
    {
        private string filePath = Path.Combine(Environment.CurrentDirectory, "mesto.dat");
        private string imagePath = Path.Combine(Environment.CurrentDirectory, "mesto.png");

        public void WriteFile(Town town)
        {
            using (StreamWriter writer = new StreamWriter(filePath, false))
            {
                WriteNodes(writer, town.GetNodes());
                WriteEdges(writer, town);
            }
        }

        public void SaveImage(Town town)
        {
            town.Bitmap.Save(imagePath);
        }


        private void WriteMap(StreamWriter writer, Tile[,] tiles)
        {
            for (int y = 0; y < tiles.GetLength(1); y++)
            {
                for (int x = 0; x < tiles.GetLength(0); x++)
                {
                    string str = tiles[x, y].TileType == TileType.Road ? "X" : "O";
                    writer.Write($"{str} ");
                }
                writer.WriteLine();
            }
        }

        private void WriteNodes(StreamWriter writer, List<Node> nodes)
        {
            writer.WriteLine(nodes.Count);
            foreach (Node node in nodes)
            {
                writer.WriteLine($"[{node.X};{node.Y}]");
            }
        }

        private void WriteEdges(StreamWriter writer, Town town)
        {
            List<Node> nodes = town.GetNodes();
            List<HashSet<Edge>> listOfEdges = town.FindEdges();
            int count = 0;
            foreach (HashSet<Edge> edges in listOfEdges)
                count += edges.Count;
            writer.WriteLine(count);
            foreach (HashSet<Edge> edges in listOfEdges)
            {
                foreach (Edge edge in edges)
                {
                    writer.Write($"{nodes.IndexOf(edge.Origin)}({edge.Distance}){nodes.IndexOf(edge.Target)}  ");
                }
                writer.WriteLine();
            }
        }

    }
}
